# Aliens on My Lawn
# Copyright (C) 2009 Jeffrey Armstrong (a.k.a. PrintStar)
# http://jeff.rainbow-100.com/
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# A full version of the license terms is available in LICENSE.

from pygame import *
from base_sprite import BaseSprite
from utilities import *
from constants import *
import random
from quadrant import locked_quadrant, LOCKED_QUADS
import os.path

class GameAliens:
    def __init__(self,count):
        
        self.blockers = None
        
        # Initialize alien storage
        total = 0
        avg = count/(QUAD_WIDTH*QUAD_HEIGHT-LOCKED_QUADS)
        self.sprite_quads = []
        
        # These two loops should ensure that all sprite groups are created
        for i in range(0,QUAD_WIDTH):
            row = []
            for j in range(0,QUAD_HEIGHT):
                row.append(sprite.RenderUpdates())
            self.sprite_quads.append(row)
        
        # Now add aliens separately
        for i in range(0,QUAD_WIDTH):
            for j in range(0,QUAD_HEIGHT):
                
                here = max(0,int(round(random.gauss(avg,2))))
                here = min(count-total,here)
                if locked_quadrant((i,j)):
                    here = 0
                total = total + here
                for alien_i in range(0,here):
                    self.sprite_quads[i][j].add(Alien())
                
        # Add any remaining ones at 1,3
        if total < count:
            for i in range(0,count-total):
                try:
                    self.sprite_quads[1][GRID_HEIGHT-1].add(Alien())
                except IndexError:
                    pass
                
    def aliens_for_quadrant(self,quad):
        # print quad
        return self.sprite_quads[quad[0]][quad[1]]
    
    def count(self):
        count = 0
        for r in self.sprite_quads:
            for ru in r:
                count = count + len(ru)
        return count

def moving_tgt():
    return 75+450*random.uniform(0.0,1.0)

class Alien(BaseSprite):
    def __init__(self):
        BaseSprite.__init__(self)
        self.velocity = 2
        
        self.init_position()
        
        self.imagedb = []
        self.build_imagedb()
        
        self.moved_this_direction = 0
        self.moving_target = moving_tgt()
        
    def build_imagedb(self):
        s = []
        for dir in ['n','s','e','w']:
            s = []
            for i in range(1,51):
                s.append(os.path.join('alien-%s' % (dir,),str(i).zfill(4)+'.png'))
            for i in range(50,1,-1):
                s.append(os.path.join('alien-%s' % (dir,),str(i).zfill(4)+'.png'))
                
            self.imagedb.append(s)
        
    def check_direction(self):
        ae = at_edge(self)
        if ae >= 0:
            self.direction = opposite_dir(self.direction)
            self.moved_this_direction = 0
            self.moving_target = moving_tgt()
            return
        
        if self.moved_this_direction > self.moving_target:
            self.direction = random.randint(0,3)
            self.moved_this_direction = 0
            self.moving_target = moving_tgt()
            return
        
    def init_position(self,avoid=None):
        acceptable=False
        while(not acceptable):
            gxy = random_grid()
            self.rect.topleft = grid2xy(gxy)
            
            self.direction = random.randint(0,3)
            if gxy[0] <= 1:
                self.direction = EAST
            elif gxy[0] >= GRID_WIDTH-2:
                self.direction = WEST
            elif gxy[1] <= 1:
                self.direction = NORTH
            elif gxy[1] >= GRID_HEIGHT-2:
                self.direction = SOUTH
                
            if avoid == None:
                acceptable = True
            else:
                acceptable = not sprite.spritecollideany(self,avoid)
            
    def store_blockers(self,blockers):
        self.blockers = blockers
        
    def image_filenames(self,direction):
        try:
            return self.imagedb[direction]
        except AttributeError:
            self.imagedb = []
            self.build_imagedb()
            return self.image_filenames(direction)
    
    def update(self):
        BaseSprite.update(self)
        self.check_direction()
        self.move(self.direction, blockers=self.blockers)
        self.moved_this_direction = self.moved_this_direction + self.velocity