# Aliens on My Lawn
# Copyright (C) 2009 Jeffrey Armstrong (a.k.a. PrintStar)
# http://jeff.rainbow-100.com/
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# A full version of the license terms is available in LICENSE.


import data
import pygame
from pygame import *
from constants import *
from quadrant import *
from player import Player
from utilities import grid2xy, at_edge, new_quad_player, opposite_dir
from alien import *
from level import Level
from fullscreen import MenuSprite, DeadSprite
from sound import GameSound
from score import ScoreBoard
from bullets import AmmoSource

def main():
    
    print ' * Initializing pygame'
    pygame.init()
    
    # Sound init
    sound = True
    if pygame.mixer and not pygame.mixer.get_init():
        print 'Warning: no sound...'
        pygame.mixer = None
        sound = False
    else:
        pygame.mixer.init()
    sounds = GameSound(sound)
    
    
    screen = pygame.display.set_mode((SCR_WIDTH,SCR_HEIGHT))
    pygame.display.set_caption(TITLE)
    
    bkgd = Surface((SCR_WIDTH,SCR_HEIGHT))
    bkgd = bkgd.convert()
    bkgd.fill(Color('black'))
    
    # Load sounds
    print ' * Loading sounds'
    if sound:
        pass
    
    # Timing...
    print ' * Initializing clock'
    clock = pygame.time.Clock()
    
    # Quadrants
    print ' * Loading playing fields'
    quads = []
    for i in range(0,QUAD_WIDTH):
        row = []
        for j in range(0,QUAD_HEIGHT):
            row.append(Quadrant(bkgd,[i,j]))
        quads.append(row)
        
    # Player
    print ' * Creating player'
    player = Player()
    player.rect.topleft = grid2xy(PLAYER_GRID_START)
    player_sprites = pygame.sprite.RenderUpdates(player)
    
    # Initialize aliens
    print ' * Creating Aliens'
    aliens = GameAliens(5)
    alien_sprites = aliens.aliens_for_quadrant(PLAYER_QUAD_START)
    # print aliens.count()

    # Bullet sprites
    print ' * Initializing bullet storage'
    bullet_sprites = pygame.sprite.RenderUpdates()
    
    # Initial level
    print ' * Level setup'
    level = Level(1)
    
    # Full screen sprites
    print ' * Full screen sprite group'
    full_screen_sprites = pygame.sprite.RenderUpdates() 
    
    # ScoreBoard
    print ' * Scoreboard initializing'
    scoreboard = ScoreBoard(player)
    score_sprites = pygame.sprite.RenderUpdates(scoreboard) 
    
    # Ammo source
    print ' * Setting up ammo source'
    ammo = AmmoSource()
    ammo_src_sprites = pygame.sprite.RenderUpdates() 
    
    # Menu
    print ' * Main menu'
    main_menu = MenuSprite()
        
    quad_now = PLAYER_QUAD_START       
    quad_update = True
    run_game = True
    shooting = False
    directions = [False,False,False,False]
    last_shot = 0
    level_up = False
    menu_mode = True
    message_mode = False
    level_message = False
    while run_game:
        clock.tick(60)
        current_time = pygame.time.get_ticks()
        
        player_sprites.clear(screen, bkgd)
        alien_sprites.clear(screen, bkgd)
        bullet_sprites.clear(screen, bkgd)
        ammo_src_sprites.clear(screen,bkgd)
        
        full_screen_sprites.clear(screen,bkgd)
        
        pe = at_edge(player)
        if pe >= 0:
            last_quad = quad_now
            if pe == NORTH:
                quad_now = [quad_now[0], min(quad_now[1]+1,QUAD_HEIGHT-1)]
            elif pe == SOUTH:
                quad_now = [quad_now[0], max(quad_now[1]-1,0)]
            elif pe == EAST:
                quad_now = [min(quad_now[0]+1,QUAD_WIDTH-1),quad_now[1]]
            elif pe == WEST:
                quad_now = [max(quad_now[0]-1,0),quad_now[1]]
            
            if last_quad == quad_now:
                new_quad_player(player,opposite_dir(pe))
            else:    
                new_quad_player(player,pe)
                quad_update = True
            
            # print quad_now
        
        if menu_mode and not full_screen_sprites.has(main_menu):
            full_screen_sprites.add(main_menu)
            sounds.menu()
        
        if aliens.count() <= 0:
            level_up = True
        
        if level_up:
            level = Level(level.number+1)
            
            menu_mode = False
            level_message = True
            
            
            quad_now = PLAYER_QUAD_START
            player.rect.topleft = grid2xy(PLAYER_GRID_START)
            quad_update = True
            level_up = False
            
            aliens = GameAliens(level.aliens)
            alien_sprites = aliens.aliens_for_quadrant(quad_now)
            
            scoreboard.level = level.number
            
            shooting = False
            directions = [False,False,False,False]
        
        if level_message:
            message_mode = True
            full_screen_sprites.add(level.sprite())
            level_message = False
            pygame.event.clear()
        
        if quad_update:
            quads[quad_now[0]][quad_now[1]].draw()
            screen.blit(bkgd,(0,0))
            pygame.display.flip()
            quad_update = False
            
            # Alien sprite update
            alien_sprites = aliens.aliens_for_quadrant(quad_now)
            
            # Clear bullets
            bullet_sprites.empty()
            
            # Reposition aliens
            quads[quad_now[0]][quad_now[1]].position_aliens(alien_sprites)
            
            # Ammo source?
            if quad_now[0] == PLAYER_QUAD_START[0] and quad_now[1] == PLAYER_QUAD_START[1]:
                ammo_src_sprites.add(ammo)
            else:
                ammo_src_sprites.empty()

        for event in pygame.event.get():
            if event.type == pygame.QUIT: 
                run_game = False
            
            elif event.type == KEYUP and not menu_mode and not message_mode:
                if event.key == K_UP:
                    directions[NORTH] = False
                elif event.key == K_DOWN:
                    directions[SOUTH] = False
                elif event.key == K_RIGHT:
                    directions[EAST] = False
                elif event.key == K_LEFT:
                    directions[WEST] = False
                elif event.key == K_SPACE:
                    shooting = False
                elif event.key == K_ESCAPE:
                    menu_mode = True
                    
            elif event.type == KEYDOWN and not menu_mode and not message_mode:
                if event.key == K_UP:
                    directions[NORTH] = True
                elif event.key == K_DOWN:
                    directions[SOUTH] = True
                elif event.key == K_RIGHT:
                    directions[EAST] = True
                elif event.key == K_LEFT:
                    directions[WEST] = True
                elif event.key == K_SPACE:
                    shooting = True
                    
            elif event.type == KEYDOWN and not menu_mode and message_mode:
                if event.key == K_SPACE:
                    sounds.choice()
                    full_screen_sprites.empty()
                    message_mode = False
                    if player.dead():
                        player = Player()
                        scoreboard.player = player
                        player.rect.topleft = grid2xy(PLAYER_GRID_START)
                        menu_mode = True
                        player_sprites.empty()
                        player_sprites.add(player)
                        level = Level(1)
                        scoreboard.level = level.number
                        shooting = False
                        directions = [False,False,False,False]
                        
                        quad_now = PLAYER_QUAD_START
                        quad_update = True
                
                    
            elif event.type == KEYDOWN and menu_mode and not message_mode:
                if event.key == K_SPACE:
                    sounds.choice()
                    if main_menu.exit():
                        run_game = False
                    elif main_menu.game():
                        if not main_menu.resume:
                            level_message = True
                        
                        main_menu.resume = True
                        menu_mode = False
                        full_screen_sprites.empty()
                elif event.key == K_UP:
                    main_menu.up()
                elif event.key == K_DOWN:
                    main_menu.down()
                    
                    
        if shooting == True and current_time - last_shot > TIME_BETWEEN_SHOTS:
            last_shot = current_time
            bullet = player.fire()
            if bullet != None:
                sounds.shot()
                bullet_sprites.add(bullet)
                # print aliens.count()
        
        for bullet in bullet_sprites:
            if bullet.invalid():
                bullet_sprites.remove(bullet)
        
        # Bullet Collision
        if len(pygame.sprite.groupcollide(bullet_sprites,alien_sprites,True,True)) > 0:
            sounds.pop()
        pygame.sprite.groupcollide(bullet_sprites,quads[quad_now[0]][quad_now[1]].inanimate_sprites,True,False)
        
        player.move_all_directions(directions,quads[quad_now[0]][quad_now[1]].inanimate_sprites)
        
        if pygame.sprite.spritecollideany(player,ammo_src_sprites):
            player.weapon().reset()
        
        # Alien Collision
        if pygame.sprite.spritecollideany(player,alien_sprites) and not player.is_invincible():
            player.killed()
            player.set_invincible(current_time)
            sounds.killed()
            
        if player.is_invincible():
            player.update_invincible(current_time)
        
        if not (menu_mode or message_mode):
            player_sprites.update()
            alien_sprites.update()
            bullet_sprites.update()
        else:
            full_screen_sprites.update()
        
        score_sprites.update()
        
        # Are we dead?
        if player.dead():
            message_mode = True
            full_screen_sprites.add(DeadSprite())
            main_menu.resume = False
            pygame.event.clear()
        
        # Perform updates after map redraw
        dirty_sprites = ammo_src_sprites.draw(screen) + \
                        player_sprites.draw(screen) + \
                        alien_sprites.draw(screen) + \
                        bullet_sprites.draw(screen) + \
                        full_screen_sprites.draw(screen) + \
                        score_sprites.draw(screen)
                         
        pygame.display.update(dirty_sprites)
        
        