# Aliens on My Lawn
# Copyright (C) 2009 Jeffrey Armstrong (a.k.a. PrintStar)
# http://jeff.rainbow-100.com/
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# A full version of the license terms is available in LICENSE.

from pygame import *
from weapon import *
from constants import *
from data import *
from base_sprite import BaseSprite

MOVE_PER_ROUND = 10

IMAGE_FILENAMES = [['player-n1.png'], \
                   ['player-s1.png'], \
                   ['player-e1.png'], \
                   ['player-w1.png']  ]

GUN_LOCATION = 35
INVINCIBLE = 2000

class Player(BaseSprite):
    def __init__(self):
        BaseSprite.__init__(self)
        self.weapons = [Shotgun()]
        self.weapon_index = 0
        
        self.velocity = 5
        
        self.lives = 3
        self.invincible_time = -1
  
    def weapon(self):
        return self.weapons[self.weapon_index]
  
    def set_invincible(self,clock):
        self.invincible_time = clock
    
    def update_invincible(self,clock):
        if clock-self.invincible_time > INVINCIBLE:
             self.invincible_time = -1
             self.set_image()
  
    def is_invincible(self):
        return self.invincible_time > 0
  
    def image_filenames(self,direction):
        return IMAGE_FILENAMES[direction]
  
    def killed(self):
        self.lives = self.lives - 1
        
    def dead(self):
        return self.lives <= 0
  
    def fire(self):
        bullet = self.weapons[self.weapon_index].fire()
        if bullet == None:
            return None
        else:
            
            position = self.rect.topleft
            if self.direction == NORTH:
                bullet.rect.center = (self.rect.left+GUN_LOCATION,self.rect.top)
            elif self.direction == SOUTH:
                bullet.rect.center = (self.rect.right-GUN_LOCATION,self.rect.bottom)
            elif self.direction == EAST:
                bullet.rect.center = (self.rect.right,self.rect.top+GUN_LOCATION-8)
            elif self.direction == WEST:
                bullet.rect.center = (self.rect.left,self.rect.top+GUN_LOCATION-8)
            
            bullet.direction(self.direction)
                
            return bullet
    
    def can_move(self):
        return (self.moves_this_round < MOVE_PER_ROUND)
        
    def update(self):
        BaseSprite.update(self)
        
        if self.is_invincible():
            self.image = self.image.copy()
            self.image.set_alpha(175)
        