# Aliens on My Lawn
# Copyright (C) 2009 Jeffrey Armstrong (a.k.a. PrintStar)
# http://jeff.rainbow-100.com/
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
# A full version of the license terms is available in LICENSE.

from data import *
from pygame import *
from utilities import *
from constants import *
from inanimate import InanimateSprite


TILES = ['grass.png','cement.png']

LOCKED_QUADS = 1

def layout_filename(coords):
    return "%d-%d.txt" % (coords[0], coords[1])

def locked_quadrant(coords):
    return (coords[0] == 1 and coords[1] == 1)

class Quadrant:
    def __init__(self,screen,coords):
        self.screen = screen
        self.coords = coords
        
        self.inanimate_sprites = sprite.RenderUpdates()
        
        self.load_layout()
            
    def load_layout(self):
        self.images = []
        for filename in TILES:
            self.images.append(load_image(filename,0))
        
        self.layout = []
        
        fp = load(layout_filename(self.coords),"r")
        linecount = 0
        for line in fp:
            linecount = linecount + 1
            if linecount <= GRID_HEIGHT:
                line = line.strip()
                intline = []
                for i in range(0,len(line)):
                    intline.append(int(line[i]))
                self.layout.append(intline)
            else:
                # Inaminate sprite names and locations
                elements = line.strip().split(',')
                #print line
                #print elements
                #print self.coords
                if len(elements) == 3:
                    
                    ias = InanimateSprite(grid=(int(elements[1]),int(elements[2])),image=elements[0])
                    self.inanimate_sprites.add(ias)
                
        
        fp.close()

    def position_aliens(self,alien_sprites):
        for alien in alien_sprites:
            alien.init_position(avoid=self.inanimate_sprites)
            alien.store_blockers(self.inanimate_sprites)
        
    def draw(self):
        self.screen.fill(Color('grey'))
        for j in range(0,len(self.layout)):
            row = self.layout[j]
            for i in range(0,len(row)):
                tl = grid2xy([i,j])
                self.screen.blit(self.images[row[i]],Rect(tl[0],tl[1],0,0))
                # draw.rect(self.screen,Color('red'),Rect(tl[0],tl[1],48,48),2)
        
        for obj in self.inanimate_sprites:
            self.screen.blit(obj.image,obj.rect)
        